const Extension = imports.misc.extensionUtils.getCurrentExtension();
const Convenience = Extension.imports.convenience;
const Gettext = imports.gettext;
const _ = Gettext.domain("panel-docklet").gettext;

const Lang = imports.lang;
const St = imports.gi.St;
const Clutter = imports.gi.Clutter;

const Gtk = imports.gi.Gtk;
const GObject = imports.gi.GObject;
const Params = imports.misc.params;

const Gio = imports.gi.Gio;
const GLib = imports.gi.GLib;

const DOCK_POS = {PANEL_LEFT:0, PANEL_MIDDLE_LEFT:1, PANEL_MIDDLE_RIGHT:2, PANEL_RIGHT:3, TOP:10, RIGHT:11, BOTTOM:12, LEFT:13},
	MIDDLE_CLICK = {CLOSE:0, MINIMIZE:1, NEW:2, QUIT: 3},
	MIN_OVERLAP_DIFF = 15,
	ICON_MIN_SIZE = 12,
	MIN_LABEL_LENGTH = 20;

const PRESET_DOCK = {
	"dock-orientation": DOCK_POS.LEFT,
	"dock-align": St.Align.MIDDLE,
	"filter-workspaces": false,
	"only-apps": true,
	"icon-size": 41,
	"icon-ws-indicator": true,
	"icons-overhang": 20,
	"hover-menu": true,
}
const PRESET_PANEL_DOCKLET = {
}
const PRESET_BOTTOM_PANEL = {
	"autohide": false,
	"dock-orientation": DOCK_POS.BOTTOM,
	"dock-fixed-size": true,
	"affect-strut": true,
	"dock-position": 0,
	
	"icon-size": 30,
	"icon-label": true,
	"favorites": true,
	"favorites-has-label": false,
	"workspace-indicator": true,
	"workspace-indicator-align": St.Align.END,
}

let Settings,
	Monitors,
	docklets = [];

const Dock_widget = new GObject.Class({
	Name: "Dock_widget",
	GTypeName: "Dock_widget",
	Extends: Gtk.Box,

	dock_id: 0,
	_parent: null,
	_notebook: null,
	_rownum: 0,
	
	_init: function(id, p_notebook, params) {
		this.parent({orientation:Gtk.Orientation.VERTICAL});
		this._parent = p_notebook;
		this.dock_id = id;
		
		this.margin = this.row_spacing = this.column_spacing = 10;
		this.column_homogeneous = true;
		
		
		this._notebook = new Gtk.Notebook({vexpand:true});
		this.add(this._notebook);
		
		let raw = Settings.raw,
			line = new Gtk.Box({halign:Gtk.Align.END}),
			del = new Gtk.ToolButton({
				icon_name: Gtk.STOCK_DELETE
			}),
			del_label = new Gtk.Label({label: _("Delete")}),
			label =  new Gtk.Label({use_markup: true, label: this.create_name(), xalign:0});
		line.add(del);
		line.add(del_label);
		this.add(line);
		
		del.connect("clicked", Lang.bind(this, this._delete_dock));
		
		p_notebook.insert_page(this, label, id);
		
		this.fill_settings();
		this.show_all();
	},
	
	create_scroller: function(label) {
		let scroller = new Gtk.ScrolledWindow({min_content_height: Math.min(Monitors[0][3]-200, 500)}),
			grid = this.create_grid();
		scroller.add(grid);
		this._notebook.append_page(scroller, new Gtk.Label({label: label}));
		return grid;
	},
	create_grid: function() {
		let grid = new Gtk.Grid({
				row_spacing: 5,
				column_spacing:5,
				margin: 5,
				column_homogeneous: true
			});
		grid._rownum = 0;
		
		return grid;
	},
	create_frame: function(grid, title) {
		let frame = new Gtk.Frame({
				//label: title,
				label_widget: new Gtk.Label({
						label: "<span font_weight='bold' font_size='large' color='blue'>"+title+"</span>",
						use_markup: true
					}),
				label_xalign:0.3,
				margin: 5
			}),
			new_grid = this.create_grid();
		
		grid.attach(frame, 0, grid._rownum++,  2, 1);
		frame.add(new_grid);
		return new_grid;
	},
	create_submenu: function() {
		let box = new Gtk.Grid({
				row_spacing: 0,
				column_spacing:1,
				margin_left: 20,
				margin_bottom:10,
				column_homogeneous: true
			});
		box._rownum = 0;
		
		return box;
	},
	insert_submenu: function(grid, box) {
		grid.attach(box, 0, grid._rownum++, 2, 1);
	},

	page_dock: function() {
		let grid = this.create_scroller(_("Dock")),
			box_orientation = this.create_submenu(grid),
			box_size = this.create_submenu(grid),
			box_hide = this.create_submenu(grid),
			frame_behavior = this.create_frame(grid, _("Behavior")),
			frame_position = this.create_frame(grid, _("Position")),
			frame_size = this.create_frame(grid, _("Size"));
			
		//Behavior
		//
		this.add_boolean(frame_behavior, _("Autohide"), "autohide", box_hide);
			this.add_spin(box_hide, _("Miliseconds until hiding starts"), "hide-timeout-ms", {lower:0, upper:100000});
			this.add_spin(box_hide, _("Opacity in hidden-mode"), "hide-opacity", {lower:0, upper:255});
			this.insert_submenu(frame_behavior, box_hide);
		
		
		//Position
		//
		let combo_orientation = this.add_combo(frame_position, _("Dock-orientation"), "dock-orientation", [
				[_("panel - left"), DOCK_POS.PANEL_LEFT],
				[_("panel - before date"), DOCK_POS.PANEL_MIDDLE_LEFT],
				[_("panel - after date"), DOCK_POS.PANEL_MIDDLE_RIGHT],
				[_("panel - right"), DOCK_POS.PANEL_RIGHT],
				[_("top"), DOCK_POS.TOP],
				[_("right"), DOCK_POS.RIGHT],
				[_("bottom"), DOCK_POS.BOTTOM],
				[_("left"), DOCK_POS.LEFT]
			], function(v) {
				switch(v) {
					case DOCK_POS.TOP: case DOCK_POS.RIGHT: case DOCK_POS.BOTTOM: case DOCK_POS.LEFT:
						return true;
					default:
						return false;
				}
			}, box_orientation);
		this.add_desc(frame_position, _("Sets where the docklet should be inserted on the screen."));
		this.add_combo(box_orientation, _("Dock-align"), "dock-align", [
				[_("left"), St.Align.START],
				[_("center"), St.Align.MIDDLE],
				[_("right"), St.Align.END]
			]);
			let position_scale = this.add_scale(box_orientation, _("Position"), "dock-position");
			let combo_monitor = this.add_combo(box_orientation, _("Monitor"), "dock-monitor", create_monitor_combo_array());
			this.add_desc(box_orientation, _("The docklet will be placed on this monitor."));
			this.add_boolean(box_orientation, _("Reduces desktop-space"), "affect-strut");
			this.insert_submenu(frame_position, box_orientation);
		
		
		//Size
		//
		this.add_boolean(frame_size, _("fixed size"), "dock-fixed-size", box_size);
			let size_scale = this.add_scale(box_size, _("Dock-size"), "dock-size");
			this.add_desc(box_size, _("Width or height of docklet - depending on orientation"));
			this.insert_submenu(frame_size, box_size);
		
		
		//wrap up
		//
		let change_fu = Lang.bind(this, function() {
			let m = Monitors[parseInt(combo_monitor.get_active_id())],
				i;
			
			switch(parseInt(combo_orientation.get_active_id())) {
				case DOCK_POS.RIGHT: case DOCK_POS.LEFT:
					i = 3;
					break;
				default:
					i = 2;
			}
			position_scale.set_range(0, m[i]);
			size_scale.set_range(30, m[i]);
		});
		
		combo_monitor.connect("changed", Lang.bind(this, change_fu));
		combo_orientation.connect("changed", Lang.bind(this, change_fu));
		change_fu(combo_monitor);
	},
	page_filter: function() {
		let grid = this.create_scroller(_("Filter & Order")),
			box_monitor = this.create_submenu(grid),
			box_only_app = this.create_submenu(grid),
			frame_filtering = this.create_frame(grid, _("Filter")),
			frame_ordering = this.create_frame(grid, _("Order"));
		
		//Filtering
		//
		this.add_boolean(frame_filtering, _("Split per workspace"), "filter-workspaces");
		this.add_boolean(frame_filtering, _("Only specific monitor"), "filter-monitors", box_monitor);
			this.add_combo(box_monitor, _("Monitor"), "filter-monitors-num", create_monitor_combo_array());
			this.insert_submenu(frame_filtering, box_monitor);
		
		
		//Ordering
		//
		this.add_boolean(frame_ordering, _("One icon per application"), "only-apps", box_only_app);
			this.add_boolean(box_only_app, _("Show window-count"), "icon-num");
			this.add_desc(box_only_app, _("Shows a little label on application-icons with more than one window."));
			this.insert_submenu(frame_ordering, box_only_app);
		this.add_boolean(frame_ordering, _("Order icons"), "ordered");
			this.add_desc(frame_ordering, _("Has no effect if \"One icon per application\" is enabled."));
	},
	page_icons: function() {
		let grid = this.create_scroller(_("Icons")),
			box_label = this.create_submenu(grid),
			frame_appearance = this.create_frame(grid, _("Appearance")),
			frame_behavior = this.create_frame(grid, _("Behavior"));
		
		//Appearance
		//
		let size_spin = this.add_spin(frame_appearance, _("Icon-size"), "icon-size", {lower:ICON_MIN_SIZE, upper:100}),
			overlap_spin = this.add_spin(frame_appearance, _("Icon-overhang"), "icons-overhang", {lower:0, upper:100});
		this.add_desc(frame_appearance, _("Icons-overhang outside the docklet."));
		
		this.add_boolean(frame_appearance, _("Mark if monitor is remote"), "icon-monitor-indicator");
		this.add_desc(frame_appearance, _("Shows an arrow if the window is placed on another monitor. This option has no effect if windows are filtered by monitor (in \"")+_("Filter &amp; Order")+"\").");
		
		this.add_boolean(frame_appearance, _("Mark if workspace is remote"), "icon-ws-indicator");
		this.add_desc(frame_appearance, _("Shows an arrow if the window is placed on another than the current workspace. This option has no effect if display of window-icons are split by workspace (in \"")+_("Filter &amp; Order")+"\").");
		
		this.add_boolean(frame_appearance, _("Show window title"), "icon-label", box_label);
			this.add_spin(box_label, _("Title-length"), "label-length", {lower:MIN_LABEL_LENGTH, upper:500});
			this.insert_submenu(frame_appearance, box_label);
		
		
		//Behavior
		//
		this.add_combo(frame_behavior, _("Middle mousebutton"), "middle-click-action", [
				[_("close"), MIDDLE_CLICK.CLOSE],
				[_("minimize"), MIDDLE_CLICK.MINIMIZE],
				[_("new window"), MIDDLE_CLICK.NEW],
				[_("quit application"), MIDDLE_CLICK.QUIT],
			]);
			
		this.add_boolean(frame_behavior, _("Hover-menu"), "hover-menu");
			this.add_desc(frame_behavior, _("Opens a list with windows on mouseover - wont open if application owns less than 2 windows"));
		
		
		//wrap up
		//
		let fu = function(item) {
				overlap_spin.set_range(0, item.get_value_as_int() - MIN_OVERLAP_DIFF);
			}
		size_spin.connect("value-changed", Lang.bind(this, fu));
		fu(size_spin);
	},
	page_extras: function() {
		let grid = this.create_scroller(_("extras")),
			box_favorites = this.create_submenu(grid),
			box_ws_switcher = this.create_submenu(grid),
			box_tray_menu = this.create_submenu(grid);
		
		this.add_boolean(grid, _("Show favorites"), "favorites", box_favorites);
			this.add_boolean(box_favorites, _("favorites-icons have labels"), "favorites-has-label");
			this.add_desc(box_favorites, _("If the favorites-buttons should have a label with the application-name."));
			this.insert_submenu(grid, box_favorites);
			
		this.add_boolean(grid, _("Show Workspace-switcher"), "workspace-indicator", box_ws_switcher);
			this.add_combo(box_ws_switcher, _("Align"), "workspace-indicator-align", [
				[_("start"), St.Align.START],
				[_("end"), St.Align.END]
			]);
			this.insert_submenu(grid, box_ws_switcher);
		
		this.add_boolean(grid, _("Show tray-button"), "tray-button", box_tray_menu);
			this.add_combo(box_tray_menu, _("Align"), "tray-button-align", [
				["left", St.Align.START],
				["right", St.Align.END]
			]);
			this.insert_submenu(grid, box_tray_menu);
	},

	fill_settings: function() {
		this.page_dock();
		this.page_filter();
		this.page_icons();
		this.page_extras();
	},
	
	create_name: function() {
		let raw = Settings.raw,
			i = this.dock_id,
			pos;
		
		switch(raw["dock-orientation"][i]) {
			case DOCK_POS.PANEL_LEFT:
			case DOCK_POS.PANEL_MIDDLE_LEFT:
			case DOCK_POS.PANEL_MIDDLE_RIGHT:
			case DOCK_POS.PANEL_RIGHT:
				pos = _("Panel");
			break;
			case DOCK_POS.TOP:
				pos = _("Top");
			break;
			case DOCK_POS.RIGHT:
				pos = _("Right");
			break;
			case DOCK_POS.BOTTOM:
				pos = _("Bottom");
			break;
			case DOCK_POS.LEFT:
				pos = _("left");
		}
		
		return "M"+raw["dock-monitor"][i]+": "+pos
	},
	
	add_scale: function(grid, text, key) {
		let item = new Gtk.Scale({
				orientation: Gtk.Orientation.HORIZONTAL,
				valuePos: Gtk.PositionType.RIGHT,
				digits:0
			});
		
		item.set_range(0, 3000);
		item.set_value(Settings.raw[key][this.dock_id]);
		
		item.connect("value-changed", Lang.bind(this, function(item) {this.value_changed("ai", key, item.get_value());}));
		this.add_row(grid, text, item);
		return item;
	},
	add_boolean: function(grid, text, key, dependant) {
		let item = new Gtk.Switch({
				active: Settings.raw[key][this.dock_id],
				halign:Gtk.Align.START
			});
		item.connect("notify::active", Lang.bind(this, function(item) {
				this.value_changed("ab", key, item.get_active());
				
				if(dependant)
					this._toggle_dependant(dependant, item.get_active());
			}));
		if(dependant)
			this._toggle_dependant(dependant, item.get_active());
		this.add_row(grid, text, item);
		return item;
	},
	add_combo: function(grid, text, key, list, dep_check, dependant) {
		let item = new Gtk.ComboBoxText({halign:Gtk.Align.START}),
			value = Settings.raw[key][this.dock_id];

		for(let i=0; i<list.length; ++i) {
			item.append(list[i][1].toString(), list[i][0]);
		}
		item.set_active_id(value.toString());
		
		item.connect("changed", Lang.bind(this, function(item) {
			let v = parseInt(item.get_active_id());

			if(Settings.raw[key][this.dock_id] != v)
				this.value_changed("ai", key, v);
			if(dependant)
				this._toggle_dependant(dependant, dep_check(v));
		}));
		if(dependant)
			this._toggle_dependant(dependant, dep_check(value));
		this.add_row(grid, text, item);
		return item;
	},
	add_spin: function(grid, label, key, adjustment_properties, spin_properties) {
		adjustment_properties = Params.parse(adjustment_properties, {
				lower: 0,
				upper: 100,
				step_increment: 1
			});
		let adjustment = new Gtk.Adjustment(adjustment_properties);

		spin_properties = Params.parse(spin_properties, {
				adjustment: adjustment,
				numeric: true,
				snap_to_ticks: true,
				halign: Gtk.Align.START
			}, true);
		let spin_button = new Gtk.SpinButton(spin_properties);

		spin_button.set_value(Settings.raw[key][this.dock_id]);
		spin_button.connect("value-changed", Lang.bind(this, function(item) {
			
			let value = item.get_value_as_int();

			if(Settings.raw[key][this.dock_id] != value) {
				this.value_changed("ai", key, value);
			}
		}));

		this.add_row(grid, label, spin_button);
		return spin_button;
	},
	add_desc: function(grid, text) {
		let item = new Gtk.Label({
				label: "<span font_size='small'>"+text+"</span>",
				use_markup: true,
				margin_bottom: 15,
				halign:Gtk.Align.START,
				margin_left: 10,
				wrap: true
			});
		grid.attach(item, 0, grid._rownum++,  2, 1);
		return item;
	},
	add_row: function(grid, text, widget) {
		let label = new Gtk.Label({
				label: "<b>"+text+":</b>",
				use_markup: true,
				hexpand: true,
				margin_bottom: 15,
				halign: Gtk.Align.START
			});
			
		grid.attach(label, 0, grid._rownum, 1, 1);
		grid.attach(widget, 1, grid._rownum, 1, 1);
		++grid._rownum;
	},
	
	_toggle_dependant: function(dependant, sensitive) {
		dependant.set_sensitive(sensitive);
	},

	value_changed: function(t, k, v) {
		dbus_send(this.dock_id, "selectDocklet"); //if shell has refreshed while settings-window was open, dock looses select
		Settings.raw[k][this.dock_id] = v;
		Settings.set_value(k, GLib.Variant.new(t, Settings.raw[k]));
	},
	_delete_dock: function() {
		let cancel = new Gtk.Dialog({title:_("Delete Dock?")});
		cancel.add_button(_("Delete"), false);
		cancel.add_button(_("Cancel"), true);
		let r = cancel.run();
		cancel.destroy();
		if(r)
			return;
		
		this.remove();
		
		let keys = Settings.list_keys(),
			id = this.dock_id,
			i = keys.length,
			s = Settings,
			raw = s.raw,
			key;
		
		while(i--) {
			key = keys[i];
			if(key == "dock-count" || key == "first-use")
				continue;
			
			s.raw[key].splice(id, 1);
			s.set_value(key, GLib.Variant.new(s.get_default_value(key).get_type_string(), s.raw[key]));
		}
		s.set_int("dock-count", docklets.length);
		
		//s.sync();
		dbus_send(this.dock_id, "deleteDock");
	},
	remove: function() {
		let d = docklets;
		this._parent.remove_page(this.dock_id);
		
		d.splice(this.dock_id, 1);
		for(let i=this.dock_id, max=d.length; i<max; ++i) {
			d[i].dock_id = i;
		}
		this.destroy();
	}
});

function dbus_send(id, method) {
	Gio.DBus.session.call(
			"org.gnome.Shell",
			"/org/gnome/Shell/Extensions/panelDocklet",
			"org.gnome.Shell.Extensions.panelDocklet",
			method,
			GLib.Variant.new("(i)", [id]),
			null,
			Gio.DBusCallFlags.NONE,
			-1,
			null,
			null
		);
}
function dbus_get_sync(method) {
	try { //can happen when extension.js wasnt initialised properly
		return Gio.DBus.session.call_sync(
				"org.gnome.Shell",
				"/org/gnome/Shell/Extensions/panelDocklet",
				"org.gnome.Shell.Extensions.panelDocklet",
				method,
				null,
				null,
				Gio.DBusCallFlags.NONE,
				-1,
				null,
				null
			).deep_unpack()[0];
		}
	catch(e) {
		return [[0, 0, 1920, 1080], [0, 0, 1920, 1080], [0, 0, 1920, 1080], [0, 0, 1920, 1080]];
	}
}
function select_dock(notebook, page_widget, id) {
	if(page_widget.hasOwnProperty("dock_id"))
		dbus_send(page_widget.dock_id, "selectDocklet");
	else
		dbus_send(0, "deselectDocklet");
}
function deselect_dock(notebook, page_widget, id) {
	if(page_widget.hasOwnProperty("dock_id"))
		dbus_send(page_widget.dock_id, "deselectDocklet");
}

function create_monitor_combo_array() {
	let combo_monitor_array = [];
			
	for(let i=0, max=Monitors.length; i<max; ++i) {
		combo_monitor_array.push([_("Monitor ")+(i+1), i]);
	}
	return combo_monitor_array;
}

function create_new_dock(preset, notebook) {
	let dock_num = docklets.length,
		keys = Settings.list_keys(),
		i = keys.length,
		s = Settings,
		raw = s.raw,
		key, default_value;
	
	while(i--) {
		key = keys[i];
		if(key == "dock-count" || key == "first-use")
			continue;
		
		default_value = s.get_default_value(key);
		raw[key][dock_num] = preset.hasOwnProperty(key) ? preset[key] : default_value.deep_unpack()[0];
		s.set_value(key, GLib.Variant.new(default_value.get_type_string(), raw[key]));
	}
	
	s.set_int("dock-count", dock_num+1);
	//s.sync();
	
	
	//dbus_send(dock_num, "createDock");
	docklets[dock_num] = new Dock_widget(dock_num, notebook);
	notebook.prev_page();
}
function page_nDock(notebook) {
	let scroller = new Gtk.ScrolledWindow({min_content_height: Math.min(Monitors[0][3]-200, 500)}),
		box = new Gtk.Box({
			orientation: Gtk.Orientation.VERTICAL,
			border_width: 10
		}),
		dock = new Gtk.Button(),
		panel_docklet = new Gtk.Button(),
		bottom_panel = new Gtk.Button(),
		
		
		line = new Gtk.Box({
				halign:Gtk.Align.CENTER,
				orientation: Gtk.Orientation.HORIZONTAL
			}),
		combo = new Gtk.ComboBoxText({margin_left:5}),
		combo_entrys = create_monitor_combo_array();
	
	
	box.add(new Gtk.Label({
			label: _(
"Here you can create a new docklet. \n\
To speed up your customization, there are some presets to choose from. Just choose one and change it as you prefer. \n\
\n\
(saving the settings for a newly created docklet takes a few seconds. So please don't be confused if it doesnt show up instantly)"),
			margin_bottom:20,
			wrap:true
		}));
	
	for(let i=0; i<combo_entrys.length; ++i) {
		combo.append(combo_entrys[i][1].toString(), combo_entrys[i][0]);
	}
	combo.set_active_id("0");
	line.add(new Gtk.Label({label:_("Add to: ")}));
	line.add(combo);
	box.add(line);
		
	box.add(new Gtk.Label({
			label: "<span font_weight='bold' font_size='large'>"+_("Preset: Dock")+"</span>",
			use_markup: true,
			margin_top: 20,
			margin_bottom: 10,
		}));
	dock.set_image(new Gtk.Image({file: Extension.path+"/dock.png"}))
	dock.connect("clicked", Lang.bind(this, function() {
			//let obj = Object.create(PRESET_DOCK),
			let obj = PRESET_DOCK,
				m = parseInt(combo.get_active_id());
			obj["dock-monitor"] = m;
			obj["filter-monitors-num"] = m;
			obj["dock-position"] = Monitors[m][2]/2;
			create_new_dock(obj, notebook);
		}));
	box.add(dock);
	
	
	box.add(new Gtk.Label({
			label: "<span font_weight='bold' font_size='large'>"+_("Preset: Panel-Docklet")+"</span>",
			use_markup: true,
			margin_top:20,
			margin_bottom: 10,
		}));
	panel_docklet.set_image(new Gtk.Image({file: Extension.path+"/panel_docklet.png"}))
	panel_docklet.connect("clicked", Lang.bind(this, function() {
			let obj = PRESET_PANEL_DOCKLET,
				m = parseInt(combo.get_active_id());
			obj["dock-monitor"] = m;
			obj["filter-monitors-num"] = m;
			create_new_dock(obj, notebook);
		}));
	box.add(panel_docklet);
	
	
	box.add(new Gtk.Label({
			label: "<span font_weight='bold' font_size='large'>"+_("Preset: Bottom-Panel")+"</span>",
			use_markup: true,
			margin_top:20,
			margin_bottom: 10,
		}));
	bottom_panel.set_image(new Gtk.Image({file: Extension.path+"/bottom_panel.png"}))
	bottom_panel.connect("clicked", Lang.bind(this, function() {
			let obj = PRESET_BOTTOM_PANEL,
				m = parseInt(combo.get_active_id());
			obj["dock-monitor"] = m;
			obj["filter-monitors-num"] = m;
			obj["dock-size"] = Monitors[m][2];
			create_new_dock(obj, notebook);
		}));
	box.add(bottom_panel);
	scroller.add(box);
	
	notebook.append_page(scroller, new Gtk.Label({label: "+"}));
}

function init() {
	Settings = Convenience.getSettings();
	Settings.create_raw();
	Convenience.initTranslations();
	Monitors = dbus_get_sync("get_monitors"); //WORKAROUND: as far as I can tell, all the options to get monitor-informations are denied in prefs
}
function buildPrefsWidget() {
	let notebook = new Gtk.Notebook({show_border:false, tab_pos:0});
	for(let i=0, max=Settings.get_int("dock-count"); i<max; ++i) {
		docklets[i] = new Dock_widget(i, notebook);
	}
	page_nDock(notebook);
	
	let box = new Gtk.Box({
			orientation: Gtk.Orientation.VERTICAL,
			border_width: 10
		});
	box.add(new Gtk.Label({
			wrap: true,
			label: _(
"Thank you for using panel-docklet! \n\
\n\
I had a lot of fun creating it and I can't even remember how many sleepless nights I spend with this extension. So I really hope you found what you where looking for and it can meet all your needs! :) \n\
\n\
\n\
Since there are countless different combinations of options, it is pretty hard to test if all combinations behave as expected or in a way that makes sense. So if you encounter any problems or have any suggestions, please don't hesitate to contact me. \n\
I have to admit, sometimes I am pretty lazy in responding to emails but I always read them :) \n\
\n\
email: davidlewe@gmail.com")
		}));
	notebook.append_page(box, new Gtk.Label({label: "About"}));
	
	notebook.show_all();
	notebook.set_current_page(Settings.get_int("dock-count"));

	notebook.connect("switch-page", Lang.bind(this, select_dock));
	notebook.connect("page-removed", Lang.bind(this, deselect_dock));

	return notebook;
}